import { LitElement, html, css, CSSResultGroup } from 'lit';
import { customElement, property } from 'lit/decorators.js';
import { Spell, spellSort } from './Spell';

@customElement('dd-place')
export class Place extends LitElement {
  location: string = "place";

  @property() getCurrentSpell: any = null;

  static get properties() {
    return {
      location: {
        type: String,
      },
    };
  }

  spells: Spell[] = [];

  addSpell() {
    if (this.getCurrentSpell && typeof this.getCurrentSpell === 'function') {
      const spell = this.getCurrentSpell();
      if (spell) {
        this.spells.push(spell);
        localStorage.setItem(this.location, JSON.stringify(this.spells));
        this.requestUpdate();
      }
    }
 }

  removeSpell(name: string) {
    const index = this.spells.map(spell => spell?.name).indexOf(name);
    if (index > -1) {
      this.spells.splice(index, 1);
      localStorage.setItem(this.location, JSON.stringify(this.spells));
      this.requestUpdate();
    }
  }

  static override styles = [
    css`
      .place-spells {
        height: 70vh;
        position: relative;
        overflow: auto;
      }

      .place-spells > * {
        padding: .6em !important;
      }
    `
  ];

  override render() {
    const storedSpells = JSON.parse(localStorage.getItem(this.location) ?? '[]');
    this.spells = storedSpells;

    return html`
      <div @click="${this.addSpell}">
        <h3>${this.location}</h3>
        <div class="place-spells">
          ${this.spells.sort(spellSort).map(spell => spell ? html`<dd-spell .spellRemover="${(name: string) => this.removeSpell(name)}" name="${spell.name}" cost=${spell.cost} color="${spell.color}" category="${spell.category}" ?cantripMagic="${spell.cantripMagic ?? false}" ?reserveMagic="${spell.reserveMagic ?? false}" description="${spell.description}"></dd-spell>` : '')}
        </div>
      </div>
    `;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'dd-place': Place;
  }
}
